Attribute VB_Name = "modPoliSSP"
'========================================================
' Descrizione.....: Routines per operazioni con polinomi.
'                   Libreria SSP: Scientific Subroutine
'                                 Package.
' Nome dei Files..: modPoliSSP.bas
' Data............: 15/2/2000
' Versione........: 1.0 a 32 bits.
' Sistema.........: VB6 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://members.xoom.it/flanguasco/
'                   http://www.mclink.it/personal/MC7061/
'========================================================
'   Comprende le seguenti funzioni e subroutines:
'
'   p() = PADD(x(), y()):       ritorna la somma dei polinomi x() + y().
'   p() = PADDM(x(), F, y()):   ritorna il polinomio x() + F*y().
'   p() = PSUB(x(), y()):       ritorna la differenza dei polinomi x() - y().
'   p() = PMPY(x(), y()):       ritorna il prodotto dei polinomi x()*y().
'   p() = PDIV(x(), y(), r()):  ritorna il rapporto dei polinomi x()/y() ed
'                               il resto in r().
'   p() = PVSUB(x(), y()):      ritorna un polinomio in cui alla variabile di x()
'                               e' stato sostituito il polinomio y().
'   p() = PGCD(x(), y()):       ritorna il Massimo Comun Divisore tra il
'                               polinomio x() ed il polinomio y().
'
'   p() = PDER(x()):            ritorna un polinomio con i coefficienti
'                               della derivata di x().
'   p() = PINT(x()):            ritorna un polinomio con i coefficienti
'                               dell' integrale di x().
'   PILD X(), A, P, D:          calcola, al punto A, il valore P del polinomio
'                               x() ed il valore D della sua derivata.
'   P = PVAL(x(), A):           ritorna il valore del polinomio x() calcolato
'                               al punto A.
'
'   POLRT_SSP p(), rr(), ri():  Trova le radici, reali e complesse di un polinomio
'                               (di grado <= 36) a coefficienti reali.
'   DPRQD p(), rr(), ri():      Trova le radici, reali e complesse di un polinomio
'                               a coefficienti reali. Fornisce, inoltre, alcune
'                               informazioni aggiuntive sulle radici trovate.
'
'   Tipi:
'    p() as Double, r() as Double, rr() as Double, ri() as Double
'    x(), y() as Double
'    A as Double, D as Double, F as Double, P as Double
'
'   Nota:   Tutti i vettori, Vp(1 To N), contenenti coefficienti
'           di polinomi iniziano dall' indice 1 con Vp(1) = termine
'           costante e Vp(N) = coefficiente della piu' alta potenza
'           di x, i.e:  Vp(I) = coefficiente di x ^ (I - 1))
'
'   Liberamente tradotte (ed adattate alle mie esigenze) da:
'       PDP-10 Freeware Archives: 10-101 SSP, Version: 3, July 1973
'       Author: Sandia Laboratories (I.B.M.)
'       Revised by: H. David Todd, Wesleyan Univ., Middletown, CT
'       Source Language: FORTRAN IV     (Vedi file SSP.txt)
'
Option Explicit


Public Function PMPY(X#(), Y#()) As Double()
'
'   PURPOSE
'   MULTIPLY TWO POLYNOMIALS
'
'   DESCRIPTION OF PARAMETERS
'   Z - VECTOR OF RESULTANT COEFFICIENTS, ORDERED FROM
'       SMALLEST TO LARGEST POWER       (Viene ritornato dalla funzione).
'   IDIMZ - DIMENSION OF Z (CALCULATED)
'   X - VECTOR OF COEFFICIENTS FOR FIRST POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X (DEGREE IS IDIMX-1)
'   Y - VECTOR OF COEFFICIENTS FOR SECOND POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y (DEGREE IS IDIMY-1)
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   DIMENSION OF Z IS CALCULATED AS IDIMX + IDIMY - 1
'   THE COEFFICIENTS OF Z ARE CALCULATED AS SUM OF PRODUCTS
'   OF COEFFICIENTS OF X AND Y , WHOSE EXPONENTS ADD UP TO THE
'   CORRESPONDING EXPONENT OF Z.
'
    Dim I&, J&, K&, IDIMX&, IDIMY&
'
    IDIMX = UBound(X)
    IDIMY = UBound(Y)
    If IDIMX * IDIMY < 1 Then Err.Raise 5, "PMPY"
    ReDim z#(1 To IDIMX + IDIMY - 1)
'
    For I = 1 To IDIMX
        For J = 1 To IDIMY
            K = I + J - 1
            z(K) = X(I) * Y(J) + z(K)
        Next J
    Next I
'
    PMPY = z()
'
'
'
End Function

Public Function PDIV(X#(), Y#(), ByRef r#() _
    , Optional ByVal Tol# = 0.000001) As Double()
'
'   PURPOSE
'   DIVIDE ONE POLYNOMIAL BY ANOTHER
'
'   DESCRIPTION OF PARAMETERS
'   P - RESULTANT VECTOR OF INTEGRAL PART  (Viene ritornato dalla funzione).
'   IDIMP - DIMENSION OF P                 (Vedi note).
'   X - VECTOR OF COEFFICIENTS FOR DIVIDEND POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER. IT IS
'       REPLACED BY REMAINDER AFTER DIVISION.
'   IDIMX - DIMENSION OF X                 (Vedi note).
'   Y - VECTOR OF COEFFICIENTS FOR DIVISOR POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y
'   TOL - TOLERANCE VALUE BELOW WHICH COEFFICIENTS ARE
'       ELIMINATED DURING NORMALIZATION
'   IER - ERROR CODE. 0 IS NORMAL, 1 IS FOR ZERO DIVISOR    (Vedi note).
'
'   REMARKS
'   THE REMAINDER R REPLACES X. (Vedi note).
'   THE DIVISOR Y REMAINS UNCHANGED.
'   IF DIMENSION OF Y EXCEEDS DIMENSION OF X, IDIMP IS SET TO
'   ZERO And CALCULATION Is BYPASSED.   (Vedi note).
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   PNORM
'
'   METHOD
'   POLYNOMIAL X IS DIVIDED BY POLYNOMIAL Y GIVING INTEGER PART
'   P AND REMAINDER R SUCH THAT X = P*Y + R.
'   DIVISOR Y AND REMAINDER VECTOR GET NORMALIZED.  (Vedi note).
'
'   Note di traduzione:
'    1) I vettori iniziali X() e Y() rimangono inalterati e non
'       vengono normalizzati.
'    2) Il risultato PDIV() ed il resto R() vengono normalizzati
'       e ridimensionati.
'    3) Se il grado di Y() e' maggiore del grado di X() il vettore
'       del risultato, PDIV(), e' dimensionato come PDIV(0 To 0) e
'       nel resto, R(), viene ritornato X().
'    4) Se non vi e' resto il vettore R() e' dimensionato come R(0 To 0).
'    5) Se tutti i coefficienti di Y() sono piu' piccoli, in valore
'       assoluto, di Tol (Y IS ZERO POLYNOMIAL) viene generato un errore.
'
    Dim I&, II&, IDIMR&, IDIMY&, IDIMP&, J&, K&, Yt#(), p#()
'
    r() = X()   ' Inizialmente il resto e' uguale al dividendo.
    Yt() = Y()  ' Vettore dei coefficienti d' appoggio: Y() rimane inalterato.
'
    IDIMR = UBound(r)
    IDIMY = UBound(Yt)
'
    'IDIMP = IDIMR - IDIMY + 1
'
    Call PNORM(Yt(), IDIMY, Tol)
    If (IDIMY <= 0) Then Err.Raise 11, "PDIV"   ' Y IS ZERO POLYNOMIAL
    ReDim Preserve Yt(1 To IDIMY)
'
10  IDIMP = IDIMR - IDIMY + 1
    If IDIMP < 1 Then
        ReDim p(0 To 0)                         ' DEGREE OF DIVISOR WAS GREATER
        PDIV = p()                              ' THAN DEGREE OF DIVIDEND
        Exit Function                           '
    Else
        ReDim p#(1 To IDIMP)
    End If
'
'   START REDUCTION
'
60  IDIMR = IDIMY - 1
    I = IDIMP
70  II = I + IDIMR
    p(I) = r(II) / Yt(IDIMY)
'
'   SUBTRACT MULTIPLE OF DIVISOR
'
    For K = 1 To IDIMR
        J = K - 1 + I
        r(J) = r(J) - p(I) * Yt(K)
    Next K
    I = I - 1
    If (I > 0) Then GoTo 70
'
'   NORMALIZE REMAINDER POLYNOMIAL
'
90  Call PNORM(r(), IDIMR, Tol)
    If IDIMR > 0 Then
        ReDim Preserve r(1 To IDIMR)
    Else
        ReDim r(0 To 0)
    End If
'
    ' Normalizzazione del risultato:
    Call PNORM(p(), IDIMP, Tol)
    ReDim Preserve p(1 To IDIMP)
    PDIV = p()
'
'
'
End Function


Private Sub PNORM(X#(), ByRef IDIMX&, Optional ByVal eps# = 0.000001)
'
'   PURPOSE
'   NORMALIZE COEFFICIENT VECTOR OF A POLYNOMIAL
'
'   DESCRIPTION OF PARAMETERS
'   X - VECTOR OF ORIGINAL COEFFICIENTS, ORDERED FROM
'       SMALLEST TO LARGEST POWER. IT REMAINS UNCHANGED
'   IDIMX - DIMENSION OF X. IT IS REPLACED BY FINAL DIMENSION
'   EPS - TOLERANCE BELOW WHICH COEFFICIENT IS ELIMINATED
'
'   REMARKS
'   IF ALL COEFFICIENTS ARE LESS THAN EPS, RESULT IS A ZERO
'   POLYNOMIAL WITH IDIMX=0 BUT VECTOR X REMAINS INTACT
'
'   METHOD
'   DIMENSION OF VECTOR X IS REDUCED BY ONE FOR EACH TRAILING
'   COEFFICIENT WITH AN ABSOLUTE VALUE LESS THAN OR EQUAL TO EPS
'
1   If IDIMX <= 0 Then GoTo 4
2   If (Abs(X(IDIMX)) > eps) Then GoTo 4
    IDIMX = IDIMX - 1
    GoTo 1
4
'
'
'
End Sub

Public Function PDER(X#()) As Double()
'
'   PURPOSE
'   FIND DERIVATIVE OF A POLYNOMIAL
'
'   DESCRIPTION OF PARAMETERS
'   Y - VECTOR OF COEFFICIENTS FOR DERIVATIVE, ORDERED FROM
'       SMALLEST TO LARGEST POWER   (Viene ritornato dalla funzione).
'   IDIMY - DIMENSION OF Y (EQUAL TO IDIMX-1)
'   X - VECTOR OF COEFFICIENTS FOR ORIGINAL POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   DIMENSION OF Y IS SET AT DIMENSION OF X LESS ONE. DERIVATIVE
'   IS THEN CALCULATED BY MULTIPLYING COEFFICIENTS BY THEIR
'   RESPECTIVE EXPONENTS.
'
'   Nota:   la routine e' stata modificata per poter
'           derivare anche polinomi di grado 0.
'
    Dim I&, IDIMX&, IDIMY&, Y#(), EXPT#
'
    IDIMX = UBound(X)
'
'   TEST OF DIMENSION
    If IDIMX < 1 Then Err.Raise 5, "PDER"
'
    If IDIMX > 1 Then
1       IDIMY = IDIMX - 1
        ReDim Y#(1 To IDIMY)
'
        EXPT = 0#
        For I = 1 To IDIMY
            EXPT = EXPT + 1#
            Y(I) = X(I + 1) * EXPT
        Next I
'
    Else
        ReDim Y#(1 To 1)
        Y(1) = 0#
    End If
'
4   PDER = Y()
'
'
'
End Function

Public Function PGCD(ByRef X#(), ByRef Y#(), ByRef IDIMGCD& _
    , Optional ByVal eps# = 0.000001, Optional ByRef IER&) As Double()
'
'   PURPOSE
'   DETERMINE GREATEST COMMON DIVISOR OF TWO POLYNOMIALS
'
'   DESCRIPTION OF PARAMETERS
'   X - VECTOR OF COEFFICIENTS FOR FIRST POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'   Y - VECTOR OF COEFFICIENTS FOR SECOND POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER.
'       THIS IS REPLACED BY GREATEST COMMON DIVISOR (Viene ritornato dalla funzione).
'   IDIMY - DIMENSION OF Y
'   WORK - WORKING STORAGE ARRAY
'   EPS - TOLERANCE VALUE BELOW WHICH COEFFICIENT IS
'       ELIMINATED DURING NORMALIZATION
'   IER - RESULTANT ERROR CODE WHERE
'   IER=0 NO ERROR
'   IER=1 X OR Y IS ZERO POLYNOMIAL
'
'   REMARKS
'   IDIMX MUST BE GREATER THAN IDIMY
'   IDIMY=1 ON RETURN MEANS X AND Y ARE PRIME, THE GCD IS A
'   CONSTANT. IDIMX IS DESTROYED DURING COMPUTATION.    (Vedi note).
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   PDIV
'   PNORM
'
'   METHOD
'   GREATEST COMMON DIVISOR OF TWO POLYNOMIALS X AND Y IS
'   DETERMINED BY MEANS OF EUCLIDEAN ALGORITHM. COEFFICIENT
'   VECTORS X AND Y ARE DESTROYED AND GREATEST COMMON
'   DIVISOR IS GENERATED IN Y.  (Vedi note).
'
'   Note di traduzione:
'    1) I vettori iniziali X() e Y() rimangono inalterati.
'    2) Il risultato PGCD() e' il massimo comun divisore.
'    3) Se i due polinomi sono primi fra loro viene ritornato
'       IDIMGCD = 1.
'
' ..................................................................
'
    Dim J&, IDIMX&, IDIMY&, NDIM&, XT#(), Yt#(), WORK#()
'
    XT() = X()
    Yt() = Y()
    IDIMX = UBound(X)
    IDIMY = UBound(Y)
    If (IDIMX < 2) Then Err.Raise 5, "PGCD"
    If (IDIMX <= IDIMY) Then Err.Raise 5, "PGCD"
'
'   DIMENSION REQUIRED FOR VECTOR NAMED WORK IS IDIMX-IDIMY+1
'
1   WORK() = PDIV(XT(), Yt(), XT())
'
    IDIMX = UBound(XT)
2   On IF_ARI(IDIMX) GoTo 5, 5, 3
'
'   INTERCHANGE X AND Y
'
3   WORK() = XT()
    XT() = Yt()
    Yt() = WORK()
    GoTo 1
'
5
    IDIMGCD = UBound(Yt)
    PGCD = Yt()
'
'
'
End Function

Public Sub PILD(X#(), ByVal Argum#, ByRef Poly#, ByRef DVal#)
'
'   PURPOSE
'   EVALUATE POLYNOMIAL AND ITS FIRST DERIVATIVE FOR A GIVEN
'   ARGUMENT
'
'   DESCRIPTION OF PARAMETERS
'   POLY - VALUE OF POLYNOMIAL
'   DVAL - DERIVATIVE
'   ARGUM - ARGUMENT
'   X - VECTOR OF COEFFICIENTS FOR POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   PQSD
'
'   METHOD
'   EVALUATION IS DONE BY MEANS OF SUBROUTINE PQSD (QUADRATIC
'   SYNTHETIC DIVISION)
'
' ..................................................................
'
    Dim p#, q#
'
    p = Argum + Argum
    q = -Argum * Argum
'
    Call PQSD(X(), DVal, Poly, p, q)
'
    Poly = Argum * DVal + Poly
'
'
'
'
End Sub

Public Function PINT(X#()) As Double()
'
'   PURPOSE
'   FIND INTEGRAL OF A POLYNOMIAL WITH CONSTANT OF INTEGRATION
'   EQUAL TO ZERO
'
'   DESCRIPTION OF PARAMETERS
'   Y - VECTOR OF COEFFICIENTS FOR INTEGRAL, ORDERED FROM
'       SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y (EQUAL TO IDIMX+1)
'   X - VECTOR OF COEFFICIENTS FOR ORIGINAL POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   DIMENSION OF Y IS SET AT DIMENSION OF X PLUS ONE, AND THE
'   CONSTANT TERM IS SET TO ZERO. INTEGRAL IS THEN CALCULATED
'   BY DIVIDING COEFFICIENTS BY THEIR RESPECTIVE EXPONENTS.
'
' ..................................................................
'
    Dim I&, IDIMX&, IDIMY&, EXPT#
'
    IDIMX = UBound(X)
    If IDIMX < 1 Then Err.Raise 5, "PINT"
'
    IDIMY = IDIMX + 1
    ReDim Y#(1 To IDIMY)
'
    Y(1) = 0#
    If (IDIMX > 0) Then
        EXPT = 1#
        For I = 2 To IDIMY
            Y(I) = X(I - 1) / EXPT
            EXPT = EXPT + 1#
        Next I
    End If
'
    PINT = Y()
'
'
'
End Function

Public Sub POLRT_SSP(XCOF#(), ByRef ROOTR#(), ByRef ROOTI#(), ByRef IER&)
'
'   PURPOSE
'   COMPUTES THE REAL AND COMPLEX ROOTS OF A REAL POLYNOMIAL
'
'   DESCRIPTION OF PARAMETERS
'   XCOF -VECTOR OF M+1 COEFFICIENTS OF THE POLYNOMIAL
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   COF -WORKING VECTOR OF LENGTH M+1
'   M -ORDER OF POLYNOMIAL
'   ROOTR-RESULTANT VECTOR OF LENGTH M CONTAINING REAL ROOTS
'       OF THE POLYNOMIAL
'   ROOTI-RESULTANT VECTOR OF LENGTH M CONTAINING THE
'       CORRESPONDING IMAGINARY ROOTS OF THE POLYNOMIAL
'   IER -ERROR CODE WHERE
'       IER=0 NO ERROR
'       IER=1 M LESS THAN ONE
'       IER=2 M GREATER THAN 36
'       IER=3 UNABLE TO DETERMINE ROOT WITH 500 INTERATIONS
'             ON 5 STARTING VALUES
'       IER=4 HIGH ORDER COEFFICIENT IS ZERO
'
'   REMARKS
'   LIMITED TO 36TH ORDER POLYNOMIAL OR LESS.
'   FLOATING POINT OVERFLOW MAY OCCUR FOR HIGH ORDER
'   POLYNOMIALS BUT WILL NOT AFFECT THE ACCURACY OF THE RESULTS.
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   NEWTON-RAPHSON ITERATIVE TECHNIQUE. THE FINAL ITERATIONS
'   ON EACH ROOT ARE PERFORMED USING THE ORIGINAL POLYNOMIAL
'   RATHER THAN THE REDUCED POLYNOMIAL TO AVOID ACCUMULATED
'   ERRORS IN THE REDUCED POLYNOMIAL.
'
' ...............................................................
'
    Dim I&, ICT&, IFIT&, IN1&, ITEMP&, FI&, KJ1&, L&, M&, MT&, N&, N2&, NX&, NXX&
    Dim XO#, YO#, X#, Y#, XPR#, YPR#, UX#, UY#, v#, Yt#, XT#, u#, XT2#, YT2#, SUMSQ#
    Dim dx#, DY#, TEMP#, ALPHA#
'
    M = UBound(XCOF) - 1
    ReDim cof#(1 To M + 1), ROOTR(1 To M), ROOTI(1 To M)
'
    IFIT = 0
    N = M
    IER = 0
    On IF_ARI(XCOF(N + 1)) GoTo 10, 25, 10
10  On IF_ARI(N) GoTo 15, 15, 32
'
'   SET ERROR CODE TO 1
'
15  IER = 1
20  Exit Sub 'Return
'
'   SET ERROR CODE TO 4
'
25  IER = 4
    GoTo 20
'
'   SET ERROR CODE TO 2
'
30  IER = 2
    GoTo 20
32  On IF_ARI(N - 36) GoTo 35, 35, 30
35  NX = N
    NXX = N + 1
    N2 = 1
    KJ1 = N + 1
    For L = 1 To KJ1
        MT = KJ1 - L + 1
40      cof(MT) = XCOF(L)
    Next L
'
'   SET INITIAL VALUES
'
45  XO = 0.00500101
    YO = 0.01000101
'
'   ZERO INITIAL VALUE COUNTER
'
    IN1 = 0
50  X = XO
'
'   INCREMENT INITIAL VALUES AND COUNTER
'
    XO = -10# * YO
    YO = -10# * X
'
'   SET X AND Y TO CURRENT VALUE
'
    X = XO
    Y = YO
    IN1 = IN1 + 1
    GoTo 59
55  IFIT = 1
    XPR = X
    YPR = Y
'
'   EVALUATE POLYNOMIAL AND DERIVATIVES
'
59  ICT = 0
60  UX = 0#
    UY = 0#
    v = 0#
    Yt = 0#
    XT = 1#
    u = cof(N + 1)
    On IF_ARI(u) GoTo 65, 130, 65
65  For I = 1 To N
        L = N - I + 1
        TEMP = cof(L)
        XT2 = X * XT - Y * Yt
        YT2 = X * Yt + Y * XT
        u = u + TEMP * XT2
        v = v + TEMP * YT2
        FI = I
        UX = UX + FI * XT * TEMP
        UY = UY - FI * Yt * TEMP
        XT = XT2
70      Yt = YT2
    Next I
    SUMSQ = UX * UX + UY * UY
    On IF_ARI(SUMSQ) GoTo 75, 110, 75
75  dx = (v * UY - u * UX) / SUMSQ
    X = X + dx
    DY = -(u * UY + v * UX) / SUMSQ
    Y = Y + DY
78  On IF_ARI(Abs(DY) + Abs(dx) - 0.00001) GoTo 100, 80, 80
'
'   STEP ITERATION COUNTER
'
80  ICT = ICT + 1
    On IF_ARI(ICT - 500) GoTo 60, 85, 85
85  On IF_ARI(IFIT) GoTo 100, 90, 100
90  On IF_ARI(IN1 - 5) GoTo 50, 95, 95
'
'   SET ERROR CODE TO 3
'
95  IER = 3
    GoTo 20
100 For L = 1 To NXX
        MT = KJ1 - L + 1
        TEMP = XCOF(MT)
        XCOF(MT) = cof(L)
105     cof(L) = TEMP
    Next L
    ITEMP = N
    N = NX
    NX = ITEMP
    On IF_ARI(IFIT) GoTo 120, 55, 120
110 On IF_ARI(IFIT) GoTo 115, 50, 115
115 X = XPR
    Y = YPR
120 IFIT = 0
122 On IF_ARI(Abs(Y) - 0.0001 * Abs(X)) GoTo 135, 125, 125
125 ALPHA = X + X
    SUMSQ = X * X + Y * Y
    N = N - 2
    GoTo 140
130 X = 0#
    NX = NX - 1
    NXX = NXX - 1
135 Y = 0#
    SUMSQ = 0#
    ALPHA = X
    N = N - 1
140 cof(2) = cof(2) + ALPHA * cof(1)
145 For L = 2 To N
150     cof(L + 1) = cof(L + 1) + ALPHA * cof(L) - SUMSQ * cof(L - 1)
    Next L
155 ROOTI(N2) = Y
    ROOTR(N2) = X
    N2 = N2 + 1
    On IF_ARI(SUMSQ) GoTo 160, 165, 160
160 Y = -Y
    SUMSQ = 0#
    GoTo 155
165 On IF_ARI(N) GoTo 20, 20, 45
'
'
'
End Sub

Public Sub DPRQD(c#(), ByRef q#(), ByRef e#(), ByRef pol#(), ByRef IR&, ByRef IER&)
'
'   PURPOSE
'   CALCULATE ALL REAL AND COMPLEX ROOTS OF A GIVEN POLYNOMIAL
'   WITH REAL COEFFICIENTS.
'
'   DESCRIPTION OF PARAMETERS
'   C - COEFFICIENT VECTOR OF GIVEN POLYNOMIAL
'       COEFFICIENTS ARE ORDERED FROM LOW TO HIGH
'       THE GIVEN COEFFICIENT VECTOR GETS DIVIDED BY THE
'       LAST NONZERO TERM  (Vedi note).
'   IC - DIMENSION OF VECTOR C
'   Q - WORKING STORAGE OF DIMENSION IC
'       ON RETURN Q CONTAINS REAL PARTS OF ROOTS
'   E - WORKING STORAGE OF DIMENSION IC
'       ON RETURN E CONTAINS COMPLEX PARTS OF ROOTS
'   POL - WORKING STORAGE OF DIMENSION IC
'       ON RETURN POL CONTAINS THE COEFFICIENTS OF THE
'       POLYNOMIAL WITH CALCULATED ROOTS
'       THIS RESULTING COEFFICIENT VECTOR HAS DIMENSION IR+1
'       COEFFICIENTS ARE ORDERED FROM LOW TO HIGH
'   IR - NUMBER OF CALCULATED ROOTS
'       NORMALLY IR IS EQUAL TO DIMENSION IC MINUS ONE
'   IER - RESULTING ERROR PARAMETER. SEE REMARKS
'
'   REMARKS
'   THE REAL PART OF THE ROOTS IS STORED IN Q(1) UP TO Q(IR)
'   CORRESPONDING COMPLEX PARTS ARE STORED IN E(1) UP TO E(IR).
'   IER = 0 MEANS NO ERRORS
'   IER = 1 MEANS NO CONVERGENCE WITH FEASIBLE TOLERANCE
'   IER = 2 MEANS POLYNOMIAL IS DEGENERATE (CONSTANT OR ZERO)
'   IER = 3 MEANS SUBROUTINE WAS ABANDONED DUE TO ZERO DIVISOR
'   IER = 4 MEANS THERE EXISTS NO S-FRACTION
'   IER =-1 MEANS CALCULATED COEFFICIENT VECTOR REVEALS POOR
'           ACCURACY OF THE CALCULATED ROOTS.
'           THE CALCULATED COEFFICIENT VECTOR HAS LESS THAN
'           3 CORRECT DIGITS.
'   THE FINAL COMPARISON BETWEEN GIVEN AND CALCULATED
'   COEFFICIENT VECTOR IS PERFORMED ONLY IF ALL ROOTS HAVE BEEN
'   CALCULATED.
'   THE MAXIMAL RELATIVE ERROR OF THE COEFFICIENT VECTOR IS
'   RECORDED IN Q(IR+1).
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   THE ROOTS OF THE POLYNOMIAL ARE CALCULATED BY MEANS OF
'   THE QUOTIENT-DIFFERENCE ALGORITHM WITH DISPLACEMENT.
'   REFERENCE
'   H.RUTISHAUSER, DER QUOTIENTEN-DIFFERENZEN-ALGORITHMUS,
'   BIRKHAEUSER, BaseL / STUTTGART, 1957#
'
'   Note di traduzione:
'    1) Il vettore dei coefficienti c() rimane inalterato.
'
' ..................................................................
'
    Dim I&, IC&, IEND&, ISTA&, J&, JBEG&, JEND&, K&, KOUNT&, LIMIT&, NRAN&, NSAV&
    Dim ESAV#, EXPT#, eps#, TEps#, TDelt#, o#, p#, r#, s#, t#, u#, v#, w#, Tol#, CT#()
'
    IC = UBound(c)
    CT() = c()  ' Vettore dei coefficienti d' appoggio: c() rimane inalterato.
    ReDim q(1 To IC), e(1 To IC), pol(1 To IC)
'
'   NORMALIZATION OF GIVEN POLYNOMIAL
'   TEST OF DIMENSION
'   IR CONTAINS INDEX OF HIGHEST COEFFICIENT
    IER = 0
    IR = IC
    eps = 1E-16
    Tol = 0.000001
    LIMIT = 10 * IC
    KOUNT = 0
1   On IF_ARI(IR - 1) GoTo 79, 79, 2
'
'   DROP TRAILING ZERO COEFFICIENTS
2   On IF_ARI(CT(IR)) GoTo 4, 3, 4
3   IR = IR - 1
    GoTo 1
'
'   REARRANGEMENT OF GIVEN POLYNOMIAL
'   EXTRACTION OF ZERO ROOTS
4   o = 1# / CT(IR)
    IEND = IR - 1
    ISTA = 1
    NSAV = IR + 1
    JBEG = 1
'
'   q(J) = 1#
'   q(J + I) = c(IR - I) / c(IR)
'   q(IR) = c(J) / c(IR)
'   WHERE J IS THE INDEX OF THE LOWEST NONZERO COEFFICIENT
    For I = 1 To IR
        J = NSAV - I
        On IF_ARI(CT(I)) GoTo 7, 5, 7
5       On JBEG GoTo 6, 8
6       NSAV = NSAV + 1
        q(ISTA) = 0#
        e(ISTA) = 0#
        ISTA = ISTA + 1
        GoTo 9
7       JBEG = 2
8       q(J) = CT(I) * o
        CT(I) = q(J)
9
    Next I
'
'   INITIALIZATION
    ESAV = 0#
    q(ISTA) = 0#
10  NSAV = IR
'
'   COMPUTATION OF DERIVATIVE
    EXPT = IR - ISTA
    e(ISTA) = EXPT
    For I = ISTA To IEND
        EXPT = EXPT - 1#
        pol(I + 1) = eps * Abs(q(I + 1)) + eps
11      e(I + 1) = q(I + 1) * EXPT
    Next I
'
'   TEST OF REMAINING DIMENSION
    On IF_ARI(ISTA - IEND) GoTo 12, 20, 60
12  JEND = IEND - 1
'
'   COMPUTATION OF S-FRACTION
    For I = ISTA To JEND
        On IF_ARI(I - ISTA) GoTo 13, 16, 13
13      On IF_ARI(Abs(e(I)) - pol(I + 1)) GoTo 14, 14, 16
'
'       THE GIVEN POLYNOMIAL HAS MULTIPLE ROOTS, THE COEFFICIENTS OF
'       THE COMMON FACTOR ARE STORED FROM Q(NSAV) UP TO Q(IR)
14      NSAV = I
        For K = I To JEND
            On IF_ARI(Abs(e(K)) - pol(K + 1)) GoTo 15, 15, 80
15
        Next K
        GoTo 21
'
'       EUCLIDEAN ALGORITHM
16      For K = I To IEND
            e(K + 1) = e(K + 1) / e(I)
            q(K + 1) = e(K + 1) - q(K + 1)
            On IF_ARI(K - I) GoTo 18, 17, 18
'
'           TEST FOR SMALL DIVISOR
17          On IF_ARI(Abs(q(I + 1)) - pol(I + 1)) GoTo 80, 80, 19
18          q(K + 1) = q(K + 1) / q(I + 1)
            pol(K + 1) = pol(K + 1) / Abs(q(I + 1))
            e(K) = q(K + 1) - e(K)
19
        Next K
    Next I
20  q(IR) = -q(IR)
'
'   THE DISPLACEMENT EXPT IS SET TO 0 AUTOMATICALLY.
'   E(ISTA)=0.,Q(ISTA+1),...,E(NSAV-1),Q(NSAV),E(NSAV)=0.,
'   FORM A DIAGONAL OF THE QD-ARRAY.
'   INITIALIZATION OF BOUNDARY VALUES
21  e(ISTA) = 0#
    NRAN = NSAV - 1
22  e(NRAN + 1) = 0#
'
'   TEST FOR LINEAR OR CONSTANT FACTOR
'   NRAN - ISTA Is DEGREE - 1
    On IF_ARI(NRAN - ISTA) GoTo 24, 23, 31
'
'   LINEAR FACTOR
23  q(ISTA + 1) = q(ISTA + 1) + EXPT
    e(ISTA + 1) = 0#
'
'   TEST FOR UNFACTORED COMMON DIVISOR
24  e(ISTA) = ESAV
    On IF_ARI(IR - NSAV) GoTo 60, 60, 25
'
'   INITIALIZE QD-ALGORITHM FOR COMMON DIVISOR
25  ISTA = NSAV
    ESAV = e(ISTA)
    GoTo 10
'
'   COMPUTATION OF ROOT PAIR
26  p = p + EXPT
'
'   TEST FOR REALITY
    On IF_ARI(o) GoTo 27, 28, 28
'
'   COMPLEX ROOT PAIR
27  q(NRAN) = p
    q(NRAN + 1) = p
    e(NRAN) = t
    e(NRAN + 1) = -t
    GoTo 29
'
'   REAL ROOT PAIR
28  q(NRAN) = p - t
    q(NRAN + 1) = p + t
    e(NRAN) = 0#
'
'   REDUCTION OF DEGREE BY 2 (DEFLATION)
29  NRAN = NRAN - 2
    GoTo 22
'
'   COMPUTATION OF REAL ROOT
30  q(NRAN + 1) = EXPT + p
'
'   REDUCTION OF DEGREE BY 1 (DEFLATION)
    NRAN = NRAN - 1
    GoTo 22
'
'   START QD-ITERATION
31  JBEG = ISTA + 1
    JEND = NRAN - 1
    TEps = eps
    TDelt = 0.01
32  KOUNT = KOUNT + 1
    p = q(NRAN + 1)
    r = Abs(e(NRAN))
'
'   TEST FOR CONVERGENCE
    On IF_ARI(r - TEps) GoTo 30, 30, 33
33  s = Abs(e(JEND))
'
'   IS THERE A REAL ROOT NEXT
    On IF_ARI(s - r) GoTo 38, 38, 34
'
'   IS DISPLACEMENT SMALL ENOUGH
34  On IF_ARI(r - TDelt) GoTo 36, 35, 35
35  p = 0#
36  o = p
    For J = JBEG To NRAN
        q(J) = q(J) + e(J) - e(J - 1) - o
'
'       TEST FOR SMALL DIVISOR
        On IF_ARI(Abs(q(J)) - pol(J)) GoTo 81, 81, 37
37      e(J) = q(J + 1) * e(J) / q(J)
    Next J
    q(NRAN + 1) = -e(NRAN) + q(NRAN + 1) - o
    GoTo 54
'
'   CALCULATE DISPLACEMENT FOR DOUBLE ROOTS
'   QUADRATIC EQUATION FOR DOUBLE ROOTS
'   X**2-(Q(NRAN)+Q(NRAN+1)+E(NRAN))*X+Q(NRAN)*Q(NRAN+1)=0
38  p = 0.5 * (q(NRAN) + e(NRAN) + q(NRAN + 1))
    o = p * p - q(NRAN) * q(NRAN + 1)
    t = Sqr(Abs(o))
'
'   TEST FOR CONVERGENCE
    On IF_ARI(s - TEps) GoTo 26, 26, 39
'
'   ARE THERE COMPLEX ROOTS
39  On IF_ARI(o) GoTo 43, 40, 40
40  On IF_ARI(p) GoTo 42, 41, 41
41  t = -t
42  p = p + t
    r = s
    GoTo 34
'
'   MODIFICATION FOR COMPLEX ROOTS
'   IS DISPLACEMENT SMALL ENOUGH
43  On IF_ARI(s - TDelt) GoTo 44, 35, 35
'
'   INITIALIZATION
44  o = q(JBEG) + e(JBEG) - p
'
'   TEST FOR SMALL DIVISOR
    On IF_ARI(Abs(o) - pol(JBEG)) GoTo 81, 81, 45
45  t = (t / o) ^ 2
    u = e(JBEG) * q(JBEG + 1) / (o * (1# + t))
    v = o + u
    KOUNT = KOUNT + 2
'
'   THREEFOLD LOOP FOR COMPLEX DISPLACEMENT
    For J = JBEG To NRAN
        o = q(J + 1) + e(J + 1) - u - p
'
'       TEST FOR SMALL DIVISOR
        On IF_ARI(Abs(v) - pol(J)) GoTo 46, 46, 49
46      On IF_ARI(J - NRAN) GoTo 81, 47, 81
47      EXPT = EXPT + p
        On IF_ARI(Abs(e(JEND)) - Tol) GoTo 48, 48, 81
48      p = 0.5 * (v + o - e(JEND))
        o = p * p - (v - u) * (o - u * t - o * w * (1# + t) / q(JEND))
        t = Sqr(Abs(o))
        GoTo 26
'
'       TEST FOR SMALL DIVISOR
49      On IF_ARI(Abs(o) - pol(J + 1)) GoTo 46, 46, 50
50      w = u * o / v
        t = t * (v / o) ^ 2
        q(J) = v + w - e(J - 1)
        u = 0#
        On IF_ARI(J - NRAN) GoTo 51, 52, 52
51      u = q(J + 2) * e(J + 1) / (o * (1# + t))
52      v = o + u - w
'
'       TEST FOR SMALL DIVISOR
        On IF_ARI(Abs(q(J)) - pol(J)) GoTo 81, 81, 53
53      e(J) = w * v * (1# + t) / q(J)
    Next J
    q(NRAN + 1) = v - e(NRAN)
54  EXPT = EXPT + p
    TEps = TEps * 1.1
    TDelt = TDelt * 1.1
    On IF_ARI(KOUNT - LIMIT) GoTo 32, 55, 55
'
'   NO CONVERGENCE WITH FEASIBLE TOLERANCE
'   ERROR RETURN IN CASE OF UNSATISFACTORY CONVERGENCE
55  IER = 1
'
'   REARRANGE CALCULATED ROOTS
56  IEND = NSAV - NRAN - 1
    e(ISTA) = ESAV
    On IF_ARI(IEND) GoTo 59, 59, 57
57  For I = 1 To IEND
        J = ISTA + I
        K = NRAN + 1 + I
        e(J) = e(K)
58      q(J) = q(K)
    Next I
59  IR = ISTA + IEND
'
'   NORMAL RETURN
60  IR = IR - 1
    On IF_ARI(IR) GoTo 78, 78, 61
'
'   REARRANGE CALCULATED ROOTS
61  For I = 1 To IR
        q(I) = q(I + 1)
62      e(I) = e(I + 1)
    Next I
'
'   CALCULATE COEFFICIENT VECTOR FROM ROOTS
    pol(IR + 1) = 1#
    IEND = IR - 1
    JBEG = 1
    For J = 1 To IR
        ISTA = IR + 1 - J
        o = 0#
        p = q(ISTA)
        t = e(ISTA)
        On IF_ARI(t) GoTo 65, 63, 65
'
'       MULTIPLY WITH LINEAR FACTOR
63      For I = ISTA To IR
            pol(I) = o - p * pol(I + 1)
64          o = pol(I + 1)
        Next I
        GoTo 69
65      On JBEG GoTo 66, 67
66      JBEG = 2
        pol(ISTA) = 0#
        GoTo 69
'
'       MULTIPLY WITH QUADRATIC FACTOR
67      JBEG = 1
        u = p * p + t * t
        p = p + p
        For I = ISTA To IEND
            pol(I) = o - p * pol(I + 1) + u * pol(I + 2)
68             o = pol(I + 1)
        Next I
        pol(IR) = o - p
69
    Next J
    On IF_ARI(IER) GoTo 78, 70, 78
'
'   COMPARISON OF COEFFICIENT VECTORS, IE. TEST OF ACCURACY
70  p = 0#
    For I = 1 To IR
        On IF_ARI(CT(I)) GoTo 72, 71, 72
71      o = Abs(pol(I))
        GoTo 73
72      o = Abs((pol(I) - CT(I)) / CT(I))
73      On IF_ARI(p - o) GoTo 74, 75, 75
74      p = o
75
    Next I
    On IF_ARI(p - Tol) GoTo 77, 76, 76
76  IER = -1
77  q(IR + 1) = p
    e(IR + 1) = 0#
78  Exit Sub 'Return
'
'   ERROR RETURNS
'   ERROR RETURN FOR POLYNOMIALS OF DEGREE LESS THAN 1
79  IER = 2
    IR = 0
    Exit Sub 'Return
'
'   ERROR RETURN IF THERE EXISTS NO S-FRACTION
80  IER = 4
    IR = ISTA
    GoTo 60
'
'   ERROR RETURN IN CASE OF INSTABLE QD-ALGORITHM
81  IER = 3
    GoTo 56
'
'
'
End Sub

Public Function PSUB(X#(), Y#()) As Double()
'
'   PURPOSE
'   SUBTRACT ONE POLYNOMIAL FROM ANOTHER
'
'   DESCRIPTION OF PARAMETERS
'   Z - VECTOR OF RESULTANT COEFFICIENTS, ORDERED FROM
'       SMALLEST TO LARGEST POWER   (Viene ritornato dalla funzione).
'   IDIMZ - DIMENSION OF Z (CALCULATED)
'   X - VECTOR OF COEFFICIENTS FOR FIRST POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X (DEGREE IS IDIMX-1)
'   Y - VECTOR OF COEFFICIENTS FOR SECOND POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y (DEGREE IS IDIMY-1)
'
'   REMARKS
'   VECTOR Z MAY BE IN SAME LOCATION AS EITHER VECTOR X OR
'   VECTOR Y ONLY IF THE DIMENSION OF THAT VECTOR IS NOT LESS
'   THAN THE OTHER INPUT VECTOR
'   THE RESULTANT POLYNOMIAL MAY HAVE TRAILING ZERO COEFFICIENTS
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   DIMENSION OF RESULTANT VECTOR IDIMZ IS CALCULATED AS THE
'   LARGER OF THE TWO INPUT VECTOR DIMENSIONS. COEFFICIENTS IN
'   VECTOR Y ARE THEN SUBTRACTED FROM CORRESPONDING COEFFICIENTS
'   IN VECTOR X.
'
' ..................................................................
'
    Dim I&, IDIMX&, IDIMY&, NDIM&
'
'   TEST DIMENSIONS OF SUMMANDS
'
    IDIMX = UBound(X)
    IDIMY = UBound(Y)
    NDIM = MAX(IDIMX, IDIMY)
    If NDIM < 1 Then Err.Raise 5, "PSUB"
    ReDim z#(1 To NDIM)
'
30  For I = 1 To NDIM
        On IF_ARI(I - IDIMX) GoTo 40, 40, 60
40      On IF_ARI(I - IDIMY) GoTo 50, 50, 70
50      z(I) = X(I) - Y(I)
        GoTo 80
60      z(I) = -Y(I)
        GoTo 80
70      z(I) = X(I)
80
    Next I
'
90  PSUB = z()
'
'
'
End Function

Public Function PVAL(X#(), ByVal Arg#) As Double
'
'   PURPOSE
'   EVALUATE A POLYNOMIAL FOR A GIVEN VALUE OF THE VARIABLE
'
'   DESCRIPTION OF PARAMETERS
'   RES - RESULTANT VALUE OF POLYNOMIAL (Viene ritornato dalla funzione).
'   ARG - GIVEN VALUE OF THE VARIABLE
'   X - VECTOR OF COEFFICIENTS, ORDERED FROM SMALLEST TO
'       LARGEST POWER
'   IDIMX - DIMENSION OF X
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   EVALUATION IS DONE BY MEANS OF NESTED MULTIPLICATION
'
' ..................................................................
'
    Dim J&, IDIMX&, RES#
'
    IDIMX = UBound(X)
    If IDIMX < 1 Then Err.Raise 5, "PVAL"
    J = IDIMX
'
    RES = 0#
1   If (J > 0) Then
2       RES = RES * Arg + X(J)
        J = J - 1
        GoTo 1
    End If
'
    PVAL = RES
'
'
'
End Function

Public Function PVSUB(X#(), Y#()) As Double()
'
'   PURPOSE
'   SUBSTITUTE VARIABLE OF A POLYNOMIAL BY ANOTHER POLYNOMIAL
'
'   DESCRIPTION OF PARAMETERS
'   Z - VECTOR OF COEFFICIENTS FOR RESULTANT POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER  (Viene ritornato dalla funzione).
'   IDIMZ - DIMENSION OF Z
'   X - VECTOR OF COEFFICIENTS FOR ORIGINAL POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'   Y - VECTOR OF COEFFICIENTS FOR POLYNOMIAL WHICH IS
'       SUBSTITUTED FOR VARIABLE, ORDERED FROM SMALLEST TO
'       LARGEST POWER
'   IDIMY - DIMENSION OF Y
'   WORK1 - WORKING STORAGE ARRAY (SAME DIMENSION AS Z)
'   WORK2 - WORKING STORAGE ARRAY (SAME DIMENSION AS Z)
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   PMPY
'   PADDM
'   (PCLA)
'
'   METHOD
'   VARIABLE OF POLYNOMIAL X IS SUBSTITUTED BY POLYNOMIAL Y
'   TO FORM POLYNOMIAL Z. DIMENSION OF NEW POLYNOMIAL IS
'   (IDIMX-1)*(IDIMY-1)+1. SUBROUTINE REQUIRES TWO WORK AREAS
'
' ..................................................................
'
    Dim I&, IDIMX&, IDIMY& ', IDIMZ&, IW1&, IW2&
    Dim Fact#, z#(), WORK1#(), WORK2#()
'
    IDIMX = UBound(X)
    IDIMY = UBound(Y)
'
'   TEST OF DIMENSIONS
    If IDIMX < 2 Then Err.Raise 5, "PVSUB"
    If IDIMY < 1 Then Err.Raise 5, "PVSUB"
'
    ReDim WORK1(1 To 1)
    WORK1(1) = 1#
    ReDim z(1 To 1)
    z(1) = X(1)
'
    For I = 2 To IDIMX
        WORK2() = PMPY(Y#(), WORK1())
        WORK1() = WORK2()
        Fact = X(I)
        z() = PADDM(z(), Fact, WORK1())
    Next I
'
    PVSUB = z()
'
'
'
End Function

Public Function PADD(X#(), Y#()) As Double()
'
'   PURPOSE
'   ADD TWO POLYNOMIALS
'
'   DESCRIPTION OF PARAMETERS
'   Z - VECTOR OF RESULTANT COEFFICIENTS, ORDERED FROM
'       SMALLEST TO LARGEST POWER   (Viene ritornato dalla funzione).
'   IDIMZ - DIMENSION OF Z (CALCULATED)
'   X - VECTOR OF COEFFICIENTS FOR FIRST POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X (DEGREE IS IDIMX-1)
'   Y - VECTOR OF COEFFICIENTS FOR SECOND POLYNOMIAL,
'       ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y (DEGREE IS IDIMY-1)
'
'   REMARKS
'   VECTOR Z MAY BE IN SAME LOCATION AS EITHER VECTOR X OR
'   VECTOR Y ONLY IF THE DIMENSION OF THAT VECTOR IS NOT LESS
'   THAN THE OTHER INPUT VECTOR
'   THE RESULTANT POLYNOMIAL MAY HAVE TRAILING ZERO COEFFICIENTS
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   DIMENSION OF RESULTANT VECTOR IDIMZ IS CALCULATED AS THE
'   LARGER OF THE TWO INPUT VECTOR DIMENSIONS. CORRESPONDING
'   COEFFICIENTS ARE THEN ADDED TO FORM Z.
'
' ..................................................................
'
    Dim I&, IDIMX&, IDIMY&, NDIM&
'
'   TEST DIMENSIONS OF SUMMANDS
'
    IDIMX = UBound(X)
    IDIMY = UBound(Y)
    NDIM = MAX(IDIMX, IDIMY)
    If NDIM < 1 Then Err.Raise 5, "PADD"
    ReDim z#(1 To NDIM)
'
30  For I = 1 To NDIM
        On IF_ARI(I - IDIMX) GoTo 40, 40, 60
40      On IF_ARI(I - IDIMY) GoTo 50, 50, 70
50      z(I) = X(I) + Y(I)
        GoTo 80
60      z(I) = Y(I)
        GoTo 80
70      z(I) = X(I)
80
    Next I
'
90  PADD = z()
'
'
'
End Function
Private Function MAX(ParamArray vD() As Variant) As Variant
'
'   Implementa la funzione MAX(D1, D2, ...DN) del FORTRAN:
'
    Dim J&, J1&, J2&, vDMax As Variant
'
    J1 = LBound(vD)
    J2 = UBound(vD)
    vDMax = vD(J1)
    For J = J1 + 1 To J2
        If vDMax < vD(J) Then vDMax = vD(J)
    Next J
'
    MAX = vDMax
'
'
'
End Function

Private Function IF_ARI(ByVal vX As Variant) As Long
'
'   Da usare come:      On IF_ARI(vX) GoTo Ln1, Ln2, Ln3
'   Per implementare:   If (vX) Ln1, Ln2, Ln3
'                       che e' l' IF aritmetico del FORTRAN.
'
'   ATTENZIONE: nel programma chiamante NON USARE il :
'   dopo i numeri di linea, altrimenti si ottiene un
'   errore fatale "Unreferenced Memory Call at ...".
'
    If vX < 0 Then
        IF_ARI = 1
    ElseIf vX = 0 Then
        IF_ARI = 2
    Else
        IF_ARI = 3
    End If
'
'
'
End Function


Private Sub PQSD(X#(), ByRef a#, ByRef b#, ByRef p#, ByRef q#)
'
'   PURPOSE
'   PERFORM QUADRATIC SYNTHETIC DIVISION
'
'   DESCRIPTION OF PARAMETERS
'   A - COEFFICIENT OF Z IN REMAINDER (CALCULATED)
'   B - CONSTANT TERM IN REMAINDER (CALCULATED)
'   P - COEFFICIENT OF Z IN QUADRATIC POLYNOMIAL
'   Q - CONSTANT TERM IN QUADRATIC POLYNOMIAL
'   X - COEFFICIENT VECTOR FOR GIVEN POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   POLYNOMIAL IS DIVIDED BY THE QUADRATIC Z**2-P*Z-Q GIVING
'   THE LINEAR REMAINDER A*Z+B
'
' ..................................................................
'
    Dim J&, IDIMX&, z#
'
    IDIMX = UBound(X)
'
    a = 0#
    b = 0#
    J = IDIMX
'
1   On IF_ARI(J) GoTo 3, 3, 2
'
2   z = p * a + b
    b = q * a + X(J)
    a = z
    J = J - 1
    GoTo 1
'
3
'
'
'
End Sub

Public Function PADDM(X#(), ByVal Fact#, Y#()) As Double()
'
'   PURPOSE
'   ADD COEFFICIENTS OF ONE POLYNOMIAL TO THE PRODUCT OF A
'   FACTOR BY COEFFICIENTS OF ANOTHER POLYNOMIAL
'
'   DESCRIPTION OF PARAMETERS
'   Z - VECTOR OF RESULTANT COEFFICIENTS, ORDERED FROM
'       SMALLEST TO LARGEST POWER   (Viene ritornato dalla funzione).
'   IDIMZ - DIMENSION OF Z (CALCULATED)
'   X - VECTOR OF COEFFICIENTS FOR FIRST POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X (DEGREE IS IDIMX-1)
'   FACT - FACTOR TO BE MULTIPLIED BY VECTOR Y
'   Y - VECTOR OF COEFFICIENTS FOR SECOND POLYNOMIAL,
'   ORDERED FROM SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y (DEGREE IS IDIMY-1)
'
'   REMARKS
'   VECTOR Z MAY BE IN SAME LOCATION AS EITHER VECTOR X OR
'   VECTOR Y ONLY IF THE DIMENSION OF THAT VECTOR IS NOT LESS
'   THAN THE OTHER INPUT VECTOR
'   THE RESULTANT POLYNOMIAL MAY HAVE TRAILING ZERO COEFFICIENTS
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   DIMENSION OF RESULTANT VECTOR IDIMZ IS CALCULATED AS THE
'   LARGER OF THE TWO INPUT VECTOR DIMENSIONS. COEFFICIENT IN
'   VECTOR X IS THEN ADDED TO COEFFICIENT IN VECTOR Y MULTIPLIED
'   BY FACTOR TO FORM Z.
'
' ..................................................................
'
    Dim I&, IDIMX&, IDIMY&, NDIM&, z#()
'
'   TEST DIMENSIONS OF SUMMANDS
'
    IDIMX = UBound(X)
    IDIMY = UBound(Y)
    NDIM = MAX(IDIMX, IDIMY)
    If NDIM < 1 Then Err.Raise 5, "PADDM"
    ReDim z#(1 To NDIM)
'
30  For I = 1 To NDIM
        On IF_ARI(I - IDIMX) GoTo 40, 40, 60
40      On IF_ARI(I - IDIMY) GoTo 50, 50, 70
50      z(I) = Fact * Y(I) + X(I)
        GoTo 80
60      z(I) = Fact * Y(I)
        GoTo 80
70      z(I) = X(I)
80
    Next I
'
90  PADDM = z()
'
'
'
End Function

Private Sub PCLA(X#(), ByVal IDIMX&, ByRef Y#(), ByRef IDIMY&)
'
'   Non usata.
'   ----------
'
'   PURPOSE
'   MOVE POLYNOMIAL X TO Y
'
'   DESCRIPTION OF PARAMETERS
'   Y - VECTOR OF RESULTANT COEFFICIENTS, ORDERED FROM
'       SMALLEST TO LARGEST POWER
'   IDIMY - DIMENSION OF Y
'   X - VECTOR OF COEFFICIENTS FOR POLYNOMIAL, ORDERED
'       FROM SMALLEST TO LARGEST POWER
'   IDIMX - DIMENSION OF X
'
'   REMARKS
'   NONE
'
'   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
'   NONE
'
'   METHOD
'   IDIMY IS REPLACED BY IDIMX AND VECTOR X IS MOVED TO Y
'
' ..................................................................
'
    IDIMX = UBound(X)
    IDIMY = IDIMX
    Y() = X()
'
'    IF(IDIMX) 30,30,10
'    10 DO 20 I=1,IDIMX
'20      Y(I) = X(I)
'30  Return
'
'
'
End Sub
